@@ -86,6 +86,7 @@ package Net::SAML;
 *zx_rsa_priv_enc = *Net::SAMLc::zx_rsa_priv_enc;
 *zx_get_rsa_pub_from_cert = *Net::SAMLc::zx_get_rsa_pub_from_cert;
 *zx_rand = *Net::SAMLc::zx_rand;
+*zx_md5_crypt = *Net::SAMLc::zx_md5_crypt;
 *zx_tok_by_ns = *Net::SAMLc::zx_tok_by_ns;
 *zx_len_xmlns_if_not_seen = *Net::SAMLc::zx_len_xmlns_if_not_seen;
 *zx_enc_xmlns_if_not_seen = *Net::SAMLc::zx_enc_xmlns_if_not_seen;
@@ -296,6 +297,7 @@ package Net::SAML;
 *mk_xacml_resp = *Net::SAMLc::zxid_mk_xacml_resp;
 *mk_xacml_simple_at = *Net::SAMLc::zxid_mk_xacml_simple_at;
 *mk_az = *Net::SAMLc::zxid_mk_az;
+*mk_az_cd1 = *Net::SAMLc::zxid_mk_az_cd1;
 *mk_attribute = *Net::SAMLc::zxid_mk_attribute;
 *mk_di_query = *Net::SAMLc::zxid_mk_di_query;
 *mk_addr = *Net::SAMLc::zxid_mk_addr;
@@ -859,6 +861,8 @@ use vars qw(@ISA %OWNER %ITERATORS %BLESSEDMEMBERS);
 *swig_pdp_url_set = *Net::SAMLc::zxid_conf_pdp_url_set;
 *swig_pdp_call_url_get = *Net::SAMLc::zxid_conf_pdp_call_url_get;
 *swig_pdp_call_url_set = *Net::SAMLc::zxid_conf_pdp_call_url_set;
+*swig_xasp_vers_get = *Net::SAMLc::zxid_conf_xasp_vers_get;
+*swig_xasp_vers_set = *Net::SAMLc::zxid_conf_xasp_vers_set;
 *swig_need_get = *Net::SAMLc::zxid_conf_need_get;
 *swig_need_set = *Net::SAMLc::zxid_conf_need_set;
 *swig_want_get = *Net::SAMLc::zxid_conf_want_get;
